@@ -22,6 +22,11 @@ def sort_tables(tables):
         visitors.traverse(table, 
                             {'schema_visitor':True}, 
                             {'foreign_key':visit_foreign_key})
+
+        tuples.extend(
+            [parent, table] for parent in table._extra_dependencies
+        )
+                            
     return list(topological.sort(tuples, tables))
 
 def find_join_source(clauses, join_to):
@@ -254,7 +259,7 @@ class Annotated(object):
         self.__dict__ = element.__dict__.copy()
         self.__element = element
         self._annotations = values
-
+        
     def _annotate(self, values):
         _values = self._annotations.copy()
         _values.update(values)
@@ -265,6 +270,13 @@ class Annotated(object):
     
     def _deannotate(self):
         return self.__element
+    
+    def _compiler_dispatch(self, visitor, **kw):
+        return self.__element.__class__._compiler_dispatch(self, visitor, **kw)
+        
+    @property
+    def _constructor(self):
+        return self.__element._constructor
         
     def _clone(self):
         clone = self.__element._clone()
@@ -288,11 +300,9 @@ class Annotated(object):
 # so that the resulting objects are pickleable.
 annotated_classes = {}
 
-from sqlalchemy.sql import expression
 for cls in expression.__dict__.values() + [schema.Column, schema.Table]:
     if isinstance(cls, type) and issubclass(cls, expression.ClauseElement):
         exec "class Annotated%s(Annotated, cls):\n" \
-             "    __visit_name__ = cls.__visit_name__\n"\
              "    pass" % (cls.__name__, ) in locals()
         exec "annotated_classes[cls] = Annotated%s" % (cls.__name__)
 
